"""
에러 핸들링에 필요한 유틸,
외부 채널을 통해 에러를 모니터링할수 있도록 도와줍니다.
"""
import traceback
import pprint
from datetime import datetime


class AppError(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message

    def __str__(self):
        return repr(f'app_error => code: {self.code}, message: {self.message}')

    def to_dict(self):
        return {
            'code': self.code,
            'message': self.message
        }


SYSTEM_NO_SUCH_CMD = AppError('B00001', 'No such cmd')
IDX_OFFSET_MUST_BE_NUMBER = AppError('B00002', 'IDX_OFFSET_MUST_BE_NUMBER')
LIMIT_MUST_LESS_EQUAL_1000 = AppError('B00003', 'LIMIT_MUST_LESS_EQUAL_1000')
SHOULD_USE_S3OBJECT_S_IN_QUERY = AppError('B00004', 'Should make query like this => "SELECT * FROM S3Object s WHERE s.<field> = <value>"')
NO_SUCH_BODY_TYPE_BY_SQS = AppError('B00005', 'NO_SUCH_BODY_TYPE_BY_SQS')
Command_Prefix_not_allowed = AppError('B00006', 'Command_Prefix_not_allowed')


def make_error_description(event):
    # 에러 발생시 설명을
    tb = traceback.format_exc()
    source_ip = event.get('requestContext', {}).get('identity', {}).get('sourceIp', None)
    request_id = event.get('requestContext', {}).get('requestId', None)

    request_payload_str = pprint.pformat(event, indent=1)
    if len(request_payload_str) > 3000:
        request_payload_str = request_payload_str[:3000]  # 1000글자 제한
        request_payload_str += '... \n*See more to query via AWS Cloudwatch, Copy & Paste bellow!*'
        request_payload_str += '```fields @timestamp, @message\n' +\
                               f' | filter requestContext.requestId = "{request_id}"\n' +\
                                ' | sort @timestamp desc\n' +\
                                ' | limit 20```\n'

    description = f'>*{datetime.now()}*\n' \
                  f'>*source_ip*: {source_ip}\n' \
                  f'>*request_id*: {request_id}\n\n' \
                  f'>*요청(1000글자):*\n{request_payload_str}\n\n' \
                  f'>*에러*:\n{tb}\n\n'
    return description


if __name__ == '__main__':
    r = make_error_description({})
    print(r)




from sawsi.aws import shared
from typing import Any


class LambdaAPI:
    def __init__(self, credentials=None, region=shared.DEFAULT_REGION):
        self.boto3_session = shared.get_boto_session(credentials)
        self.lambda_client = self.boto3_session.client('lambda', region_name=region)

    def invoke(self, function_name: str, payload: Any):
        response = self.lambda_client.invoke(
            FunctionName=function_name,
            InvocationType='RequestResponse',  # 'Event' for asynchronous execution
            Payload=payload
        )

        # 응답에서 Payload 추출
        response_payload = response['Payload'].read()
        response_body = response_payload.decode('utf-8')
        return response_body



import time
import boto3
from sawsi.aws import shared
import hmac
import hashlib
import base64


def get_secret_hash(username, client_id, client_secret):
    msg = username + client_id
    dig = hmac.new(str(client_secret).encode('utf-8'),
                   msg=str(msg).encode('utf-8'),
                   digestmod=hashlib.sha256).digest()
    return base64.b64encode(dig).decode()


class CognitoAPI:
    """
    S3 를 활용하는 커스텀 ORM 클래스
    """
    def __init__(self, user_poop_id, client_id, client_secret, credentials=None, region=shared.DEFAULT_REGION):
        """
        :param credentials: {
            "aws_access_key_id": "str",
            "aws_secret_access_key": "str",
            "region_name": "str",
            "profile_name": "str",
        }
        """
        self.boto3_session = shared.get_boto_session(credentials)
        self.cache = {}
        self.client = boto3.client('cognito-idp', region_name=region)
        self.user_pool_id = user_poop_id
        self.client_id = client_id
        self.region = region
        self.client_secret = client_secret

    def sign_up(self, phone_number, username, password):
        """
        인증번호를 사용자한테 전송합니다.
        :param username:
        :param password:
        :param phone_number:
        :return:
        """
        try:
            response = self.client.sign_up(
                ClientId=self.client_id,
                SecretHash=get_secret_hash(username, self.client_id, self.client_secret),
                Username=username,
                Password=password,
                UserAttributes=[
                    {
                        'Name': 'phone_number',
                        'Value': phone_number
                    }
                ]
            )
            return response
        except self.client.exceptions.UsernameExistsException as ex:  #  이미 회원이 존재하는 경우
            # 비활성화 상태인 경우에만 다시 전송을 허용하고, 할성 상태면 그대로 에러를 래이즈
            if self.check_user_valid(phone_number):
                raise ex
            else:
                response = self.resend_confirmation_code(username)
                return response

    def check_user_valid(self, phone_number_to_search):
        response = self.client.list_users(
            UserPoolId=self.user_pool_id,
            Filter=f'phone_number = "{phone_number_to_search}"',
            Limit=60
        )
        filtered_users = response['Users']
        # 확인된 사용자만 반환
        filtered_users = [filtered_user for filtered_user in filtered_users
                          if filtered_user['UserStatus'] == 'CONFIRMED']
        print('filtered_users:', filtered_users)
        if filtered_users:
            return True
        else:
            return False

    def resend_confirmation_code(self, username):
        response = self.client.resend_confirmation_code(
            ClientId=self.client_id,
            SecretHash=get_secret_hash(username, self.client_id, self.client_secret),
            Username=username
        )
        return response

    def confirm_sign_up(self, username, code):
        response = self.client.confirm_sign_up(
            ClientId=self.client_id,
            SecretHash=get_secret_hash(username, self.client_id, self.client_secret),
            Username=username,
            ConfirmationCode=code
        )
        if response['ResponseMetadata']['HTTPStatusCode'] == 200:
            print("Signup confirmed!")
        else:
            print("Error in confirming signup!")
        return response

    def is_user_exists_by_phone(self, phone_number):
        try:
            # Search for the user based on the phone number
            response = self.client.list_users(
                UserPoolId=self.user_pool_id,
                Filter=f'phone_number="{phone_number}"',
            )
            # Check if any user is found with the given phone number
            return 'Users' in response and len(response['Users']) > 0
        except Exception as e:
            print(f"Error checking user by phone number: {e}")
            return False

    def cognito_login(self, user_name, phone_number, password):
        try:
            response = self.client.initiate_auth(
                AuthFlow='USER_PASSWORD_AUTH',
                AuthParameters={
                    'USERNAME': phone_number,
                    'PASSWORD': password,
                    'SECRET_HASH': get_secret_hash(user_name, self.client_id, self.client_secret)
                },
                ClientId=self.client_id,

            )
            status = response.get('ChallengeName')
            session = response['Session']
            return {
                'status': status,
                'session': session,
            }

        except self.client.exceptions.NotAuthorizedException as ex:
            print(ex)
            return {
                'status': 'invalid_credential',
                'session': None
            }
        except Exception as e:
            raise e

    def confirm_cognito_login(self, session, username, phone_number, sms_code):
        try:
            mfa_response = self.client.respond_to_auth_challenge(
                ClientId=self.client_id,
                ChallengeName='SMS_MFA',
                Session=session,
                ChallengeResponses={
                    'USERNAME': phone_number,
                    'SMS_MFA_CODE': sms_code,
                    'SECRET_HASH': get_secret_hash(username, self.client_id, self.client_secret)
                },
            )
        except self.client.exceptions.NotAuthorizedException as ex:
            print(ex)
            return {
                'status': 'invalid_credential'
            }
        except Exception as e:
            raise e

        if mfa_response.get('AuthenticationResult'):
            # 성공적으로 인증된 경우
            tokens = mfa_response['AuthenticationResult']
            return {
                'status': 'success',
                'access_token': tokens['AccessToken'],
                'refresh_token': tokens.get('RefreshToken'),
                'id_token': tokens['IdToken'],
                'token_type': tokens['TokenType'],
                'expires_in': tokens['ExpiresIn']
            }
        else:
            raise Exception('UNKNOWN ISSUE')

    def verify_token(self, access_token, use_cache=True):
        # Cognito Identity Provider 클라이언트 생성
        c_key = f'{access_token}{int(time.time()) // 100}'
        if use_cache:
            response = self.cache.get(c_key, None)
            if response:
                # 있으면 응답하는데, 없으면 빠이
                return response
        try:
            # GetUser API 호출하여 토큰 검증
            response = self.client.get_user(
                AccessToken=access_token
            )
            self.cache[c_key] = response
            # 검증 성공 시 사용자 정보 반환
            return response
        except self.client.exceptions.NotAuthorizedException as e:
            raise e
        except Exception as e:
            # 기타 예외 처리
            raise e


if __name__ == '__main__':
    pass


import click
import os
import shutil
from pathlib import Path

# 현재 파일의 Path 객체
current_file_path = Path(__file__)

# sawsi_boilerplate 파일이 위치한 디렉토리
boilerplate_dir = current_file_path.parent / 'sawsi_boilerplate'


@click.group()
def cli():
    pass

@cli.command()
@click.argument('project')
@click.argument('app')
@click.option('-t', '--trigger', default='agw', help='AWS Trigger option. (agw: API Gateway, sqs: Simple Queue Service, ebs: Event Bridge Scheduler)')
def init(project, app, trigger):
    # 현재 작업 디렉토리를 기준으로 앱 디렉토리 생성
    project_path = Path(os.getcwd())
    project_path.mkdir(parents=True, exist_ok=True)

    # 기본 파일 구조 생성
    create_base_dir(app, project_path, project)

    # 앱 구조 생성
    create_app_dir(app, project_path, trigger)

    click.echo(f"{project} > App {project} has been created in {project_path}")


@cli.command()
@click.argument('app')
@click.option('-t', '--trigger', default='agw', help='AWS Trigger option. (agw: API Gateway, sqs: Simple Queue Service, ebs: Event Bridge Scheduler)')
def add(app, trigger):
    # 현재 작업 디렉토리를 기준으로 앱 디렉토리 생성
    project_path = Path(os.getcwd())
    project_path.mkdir(parents=True, exist_ok=True)

    # 앱 구조 생성
    create_app_dir(app, project_path, trigger)

    click.echo(f"App {app} has been created in {project_path}")


def create_base_dir(app_name, project_path, project_name):
    target_files = [
        'api.py',
        'errs.py',
        'requirements.txt',
        'buildspec-dev.yml',
        'buildspec-prod.yml',
        'aws_iam_policy_dev.json',
        'aws_iam_policy_prod.json',
        'build_keeper.py',
        'config.py',
        'README.md',
    ]
    for target_file in target_files:
        if isinstance(target_file, str):
            source_file = boilerplate_dir / target_file
            copy_without_overwrite(source_file, project_path)
            click.echo(f"Created: {project_path}")
        elif isinstance(target_file, tuple):
            target_file = target_file[0]
            rename_to = target_file[1]
            source_file = boilerplate_dir / target_file
            copy_without_overwrite(source_file, project_path / rename_to)
            click.echo(f"Created: {project_path}")

    # {{app}} 문자열 치환
    files_contain_app_literal = ['buildspec-dev.yml', 'buildspec-prod.yml', 'build_keeper.py']
    for file in files_contain_app_literal:
        replace_app(project_path / file, app_name)

    # {{project}} 문자열 치환
    files_contain_project_literal = [
        'buildspec-dev.yml', 'buildspec-prod.yml', 'api.py',
        'aws_iam_policy_dev.json', 'aws_iam_policy_prod.json',
    ]
    for file in files_contain_project_literal:
        replace_project(project_path / file, project_name)


def create_app_dir(app_name, project_path, trigger='agw'):
    source_dir = boilerplate_dir / 'app'
    destination_dir = project_path / app_name

    # 대상 디렉토리가 이미 존재하는 경우에도 복사를 수행
    try:
        shutil.copytree(source_dir, destination_dir, dirs_exist_ok=False)
    except Exception as ex:
        print(ex)
    # {{app}} 수정
    replace_app(destination_dir / 'controller' / 'sample_login.py', app_name)
    replace_app(destination_dir / 'doc_maker.py', app_name)
    replace_app(destination_dir / 'aws_handler.py', app_name)

def replace_app(target_file, app_name):
    replace_word(target_file, '{{app}}', app_name)

def replace_project(target_file, project_name):
    replace_word(target_file, '{{project}}', project_name)

def replace_word(target_file, word, replacement):
    rfp = open(target_file, 'r')
    text = rfp.read()
    text = text.replace(word, replacement)
    rfp.close()

    wfp = open(target_file, 'w+')
    wfp.write(text)


def copy_without_overwrite(source, destination):
    """
    shutil.copy와 유사하게 파일을 복사하지만, 대상 파일이 이미 존재하는 경우 복사를 수행하지 않습니다.

    :param source: 복사할 소스 파일의 경로입니다.
    :param destination: 파일을 복사할 대상 경로입니다. 파일 이름을 포함할 수도 있고, 디렉토리만을 지정할 수도 있습니다.
    :return: 복사가 성공적으로 수행되었으면 True를, 이미 파일이 존재하여 복사하지 않았으면 False를 반환합니다.
    """
    # destination이 디렉토리인 경우, 동일한 파일 이름으로 설정
    if os.path.isdir(destination):
        destination = os.path.join(destination, os.path.basename(source))

    # 대상 파일이 이미 존재하는지 확인
    if os.path.exists(destination):
        return False
    else:
        shutil.copy(source, destination)
        print(f'Created: {destination}')
        return True



if __name__ == '__main__':
    cli()


"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""

import time
import uuid
import api
from pydantic import fields, BaseModel
from typing import Optional, Literal, TypeVar, Generic, Type
from sawsi.model.base import DynamoModel, sync


class Address(BaseModel):
    street: str
    city: str
    country: str


@sync
class User(DynamoModel):
    __dynamo__ = api.dynamo
    __create_default_index__ = True  # Optional 이며, 파티션 키 부하에 민감한 경우 전체 쿼리를 위한 인덱스를 제거하는데 사용됨.
    _table = 'user'
    _indexes = [
        ('gender', 'crt'),
        ('gender', 'name'),
    ]

    id: str = fields.Field(default_factory=lambda: str(uuid.uuid4()))
    crt:float = fields.Field(default_factory=time.time)
    ptn:str = 'user'
    name: str
    age: int
    gender: Optional[Literal["male", "female"]] = None
    address: Address

    def view(self)->dict:
        return {
            'name': self.name,
            'age': self.age
        }


if __name__ == '__main__':
    # 모델 인스턴스 생성
    address_data = {'street': '123 Main St', 'city': 'Anytown', 'country': 'USA'}
    user_data = {'name': 'John Doe', 'age': 30, 'gender': 'male', 'address': address_data}
    user = User(**user_data)
    print('user.id', user.id)

    # 모델을 딕셔너리로 변환
    user_dict = user.model_dump()
    print(user_dict)
